@@ -22,6 +22,15 @@
  *
  */
 
+/**
+ * SECTION:bluetooth-chooser
+ * @short_description: a Bluetooth chooser widget
+ * @stability: Stable
+ * @include: bluetooth-plugin.h
+ *
+ * A tree-like widget used to select Bluetooth devices.
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -35,7 +44,6 @@
 #include "bluetooth-chooser.h"
 #include "bluetooth-chooser-private.h"
 #include "gnome-bluetooth-enum-types.h"
-#include "bling-spinner.h"
 #include "bluetooth-filter-widget.h"
 
 enum {
@@ -153,16 +161,16 @@ set_search_label (BluetoothChooser *self, gboolean state)
 
 	if (priv->show_searching == FALSE) {
 		/* Just making sure */
-		bling_spinner_stop (BLING_SPINNER (priv->spinner));
+		gtk_spinner_stop (GTK_SPINNER (priv->spinner));
 		return;
 	}
 	if (state == FALSE) {
-		bling_spinner_stop (BLING_SPINNER (priv->spinner));
+		gtk_spinner_stop (GTK_SPINNER (priv->spinner));
 		gtk_widget_hide (priv->spinner);
 		gtk_label_set_text (GTK_LABEL (priv->search_label), _("No adapters available"));
 	} else {
 		gtk_widget_show (priv->spinner);
-		bling_spinner_start (BLING_SPINNER (priv->spinner));
+		gtk_spinner_start (GTK_SPINNER (priv->spinner));
 		gtk_label_set_text (GTK_LABEL (priv->search_label), _("Searching for devices..."));
 	}
 }
@@ -226,6 +234,8 @@ bluetooth_chooser_get_selected_device_data (BluetoothChooser *self, guint column
  * bluetooth_chooser_get_selected_device:
  * @self: a #BluetoothChooser widget.
  *
+ * Returns the Bluetooth address for the currently selected device.
+ *
  * Return value: the Bluetooth address for the currently selected device, or %NULL.
  **/
 gchar *
@@ -238,6 +248,8 @@ bluetooth_chooser_get_selected_device (BluetoothChooser *self)
  * bluetooth_chooser_get_selected_device_name:
  * @self: a #BluetoothChooser widget.
  *
+ * Returns the name for the currently selected device.
+ *
  * Return value: the name for the currently selected device, or %NULL.
  **/
 gchar *
@@ -250,6 +262,8 @@ bluetooth_chooser_get_selected_device_name (BluetoothChooser *self)
  * bluetooth_chooser_get_selected_device_icon:
  * @self: a #BluetoothChooser widget.
  *
+ * Returns the icon name to use to represent the currently selected device.
+ *
  * Return value: the icon name to use to represent the currently selected device, or %NULL.
  **/
 gchar *
@@ -262,6 +276,8 @@ bluetooth_chooser_get_selected_device_icon (BluetoothChooser *self)
  * bluetooth_chooser_get_selected_device_type:
  * @self: a #BluetoothChooser widget.
  *
+ * Returns the #BluetoothType of the device selected.
+ *
  * Return value: the #BluetoothType of the device selected, or '0' if unknown.
  **/
 BluetoothType
@@ -284,7 +300,9 @@ bluetooth_chooser_get_selected_device_type (BluetoothChooser *self)
  * bluetooth_chooser_get_selected_device_is_connected:
  * @self: a #BluetoothChooser widget.
  *
- * Return value: whether the selected device is conncted to this computer,
+ * Returns whether the selected device is connected to this computer.
+ *
+ * Return value: whether the selected device is connected to this computer,
  * will always be %FALSE if no devices are selected.
  **/
 gboolean
@@ -308,6 +326,8 @@ bluetooth_chooser_get_selected_device_is_connected (BluetoothChooser *self)
  * @field: The identifier for the field to get data for.
  * @value: An empty #GValue to set.
  *
+ * Returns whether the @value has been set.
+ *
  * Return value: %TRUE if the @value has been set.
  **/
 gboolean
@@ -371,6 +391,8 @@ show_confirm_dialog (BluetoothChooser *chooser, const char *name)
  * bluetooth_chooser_remove_selected_device:
  * @self: A #BluetoothChooser widget.
  *
+ * Returns whether the removal was successful.
+ *
  * Return value: %TRUE if the selected device was correctly removed.
  **/
 gboolean
@@ -414,9 +436,11 @@ bluetooth_chooser_remove_selected_device (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_model:
- * @self: A BluetoothChooser widget.
+ * @self: A #BluetoothChooser widget.
  *
- * Return value: The BluetoothChooser's GtkTreeModel.
+ * Returns the #BluetoothChooser's #GtkTreeModel.
+ *
+ * Return value: a #GtkTreeModel object.
  **/
 GtkTreeModel *
 bluetooth_chooser_get_model (BluetoothChooser *self)
@@ -428,9 +452,11 @@ bluetooth_chooser_get_model (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_type_column:
- * @self: A BluetoothChooser widget.
+ * @self: A #BluetoothChooser widget.
  *
- * Return value: A GtkTreeViewColumn pointer to the type column of the BluetoothChooser.
+ * Returns a #GtkTreeViewColumn object to the type column of the #BluetoothChooser.
+ *
+ * Return value: a #GtkTreeViewColumn object.
  **/
 GtkTreeViewColumn *
 bluetooth_chooser_get_type_column (BluetoothChooser *self)
@@ -442,9 +468,11 @@ bluetooth_chooser_get_type_column (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_treeview:
- * @self: A BluetoothChooser widget.
+ * @self: A #BluetoothChooser widget.
  *
- * Return value: The #GtkTreeView pointer for the #BluetoothChooser.
+ * Returns the #GtkTreeView object for the #BluetoothChooser.
+ *
+ * Return value: a #GtkTreeView object.
  **/
 GtkWidget *
 bluetooth_chooser_get_treeview (BluetoothChooser *self)
@@ -834,7 +862,7 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	/* The searching label */
 	priv->search_hbox = gtk_hbox_new (FALSE, 6);
 	gtk_widget_set_no_show_all (priv->search_hbox, TRUE);
-	priv->spinner = bling_spinner_new ();
+	priv->spinner = gtk_spinner_new ();
 	gtk_container_add (GTK_CONTAINER (priv->search_hbox), priv->spinner);
 	gtk_widget_show (priv->spinner);
 	priv->search_label = gtk_label_new (_("Searching for devices..."));
@@ -942,9 +970,45 @@ static void
 bluetooth_chooser_set_property (GObject *object, guint prop_id,
 					 const GValue *value, GParamSpec *pspec)
 {
-	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(object);
+	BluetoothChooser *self = BLUETOOTH_CHOOSER (object);
+	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE (object);
 
 	switch (prop_id) {
+	case PROP_DEVICE_SELECTED: {
+		const char *address;
+		char *selected;
+		GtkTreeIter iter;
+		gboolean cont;
+
+		address = g_value_get_string (value);
+		if (address == NULL) {
+			gtk_tree_selection_unselect_all (priv->selection);
+			return;
+		}
+
+		selected = bluetooth_chooser_get_selected_device (self);
+		if (g_strcmp0 (selected, address) == 0) {
+			g_free (selected);
+			return;
+		}
+		g_free (selected);
+
+		cont = gtk_tree_model_get_iter_first (priv->filter, &iter);
+		while (cont == TRUE) {
+			char *iaddress;
+			gtk_tree_model_get (priv->filter, &iter,
+					    BLUETOOTH_COLUMN_ADDRESS, &iaddress, -1);
+			if (g_strcmp0 (iaddress, address) == 0) {
+				gtk_tree_selection_select_iter (priv->selection, &iter);
+				g_free (iaddress);
+				return;
+			}
+			g_free (iaddress);
+
+			cont = gtk_tree_model_iter_next (priv->filter, &iter);
+		}
+		break;
+	}
 	case PROP_TITLE:
 		bluetooth_chooser_set_title (BLUETOOTH_CHOOSER (object), g_value_get_string (value));
 		break;
@@ -1088,7 +1152,7 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 									  "title", "The widget header title", NULL, G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_DEVICE_SELECTED, g_param_spec_string ("device-selected",
-										    "device-selected", "the Bluetooth address for the currently selected device, or %NULL", NULL, G_PARAM_READABLE));
+										    "device-selected", "the Bluetooth address for the currently selected device, or %NULL", NULL, G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_PAIRING, g_param_spec_boolean ("show-pairing",
 										  "show-pairing", "Whether to show the pairing column in the tree.", FALSE, G_PARAM_READWRITE));
@@ -1126,6 +1190,8 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
  * bluetooth_chooser_new:
  * @title: the widget header title, if %NULL, the widget header will be hidden.
  *
+ * Returns a new #BluetoothChooser widget.
+ *
  * Return value: A #BluetoothChooser widget
  **/
 GtkWidget *
